package net.alviano.app;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import net.alviano.menu.Menu;
import net.alviano.menu.MenuDescription;
import net.alviano.menu.MenuEntryAdapter;
import net.alviano.menu.Menu.Builder;
import net.alviano.safe.Automobile;
import net.alviano.safe.CasaProduttrice;
import net.alviano.safe.Concessionaria;
import net.alviano.safe.Id;
import net.alviano.safe.ModelloVeicolo;
import net.alviano.safe.Moto;
import net.alviano.safe.Prezzo;
import net.alviano.safe.Targa;
import net.alviano.safe.Veicolo;

public final class AppConcessionaria {
	private final Menu menu;
	private final Concessionaria concessionaria;
	
	public AppConcessionaria() {
		concessionaria = new Concessionaria();
		menu = new Menu.Builder(new MenuDescription("Menu della Concessionaria"))
			.withEntry(new MenuEntryAdapter("1", "Aggiungi automobile") {
				@Override
				public void onSelected() {
					aggiungiAutomobile();
				}
			})
			.withEntry(new MenuEntryAdapter("2", "Aggiungi moto") {
				@Override
				public void onSelected() {
					aggiungiMoto();
				}
			})
			.withEntry(new MenuEntryAdapter("3", "Stampa lista veicoli") {
				@Override
				public void onSelected() {
					stampaListaVeicoli();
				}
			})
			.withEntry(new MenuEntryAdapter("4", "Ordina per prezzo") {
				@Override
				public void onSelected() {
					concessionaria.ordinaVeicoliPerPrezzo();
					System.out.println("Ordinamento completato!");
				}
			})
			.withEntry(new MenuEntryAdapter("5", "Ordina per nome") {
				@Override
				public void onSelected() {
					concessionaria.ordinaVeicoliPerNome();
					System.out.println("Ordinamento completato!");
				}
			})
			.withEntry(new MenuEntryAdapter("6", "Stampa il valore totale del parco veicoli") {
				@Override
				public void onSelected() {
					stampaValoreParcoVeicoli();
				}
			})
			.withEntry(new MenuEntryAdapter("7", "Salva su file") {
				@Override
				public void onSelected() {
					salva();
				}
			})
			.withEntry(new MenuEntryAdapter("8", "Carica da file") {
				@Override
				public void onSelected() {
					carica();
				}
			})
			.withEntry(new MenuEntryAdapter("9", "Modifica veicolo") {
				@Override
				public void onSelected() {
					modificaVeicolo();
				}
			})
			.withEntry(new MenuEntryAdapter("10", "Rimuovi veicolo") {
				@Override
				public void onSelected() {
					rimuoviVeicolo();
				}
			})
			.withEntry(new MenuEntryAdapter("11", "Stampa lista case produttrici di auto") {
				@Override
				public void onSelected() {
					stampaListaCaseProduttriciDiAuto();
				}
			})
			.withEntry(new MenuEntryAdapter("12", "Stampa lista case produttrici di moto") {
				@Override
				public void onSelected() {
					stampaListaCaseProduttriciDiMoto();
				}
			})
			.withEntry(new MenuEntryAdapter("13", "Stampa lista di auto di una data casa produttrice") {
				@Override
				public void onSelected() {
					stampaListaAutoDiCasaProduttrice();
				}
			})
			.withEntry(new MenuEntryAdapter("14", "Stampa lista di moto di una data casa produttrice") {
				@Override
				public void onSelected() {
					stampaListaMotoDiCasaProduttrice();
				}
			})
			.withEntry(new MenuEntryAdapter("0", "Esci") {
				@Override
				public void onSelected() {
					System.out.println("Bye!");
				}
				
				@Override
				public boolean isExit() {
					return true;
				}
			})
			.build();
	}
	
	private Targa leggiTarga() {
		Targa targa = null;
		for(boolean done = false; !done;) {
			try {
				System.out.print("Targa: ");
				targa = new Targa(menu.readline());
				done = true;
			} catch (IOException e) {
				System.out.println("Try again");
			} catch (IllegalArgumentException e) {
				System.out.println(e.getMessage());
			}
		}
		return targa;
	}
	
	private CasaProduttrice leggiCasaProduttrice() {
		CasaProduttrice casa = null;
		for(boolean done = false; !done;) {
			try {
				System.out.print("Casa produttrice: ");
				casa = new CasaProduttrice(menu.readline());
				done = true;
			} catch (IOException e) {
				System.out.println("Try again");
			} catch (IllegalArgumentException e) {
				System.out.println(e.getMessage());
			}
		}
		return casa;
	}
	
	private ModelloVeicolo leggiModelloVeicolo() {
		ModelloVeicolo nome = null;
		for(boolean done = false; !done;) {
			try {
				System.out.print("Nome: ");
				nome = new ModelloVeicolo(menu.readline());
				done = true;
			} catch (IOException e) {
				System.out.println("Try again");
			} catch (IllegalArgumentException e) {
				System.out.println(e.getMessage());
			}
		}
		return nome;
	}

	private Prezzo leggiPrezzoVeicolo() {
		Prezzo prezzo = null;
		for(boolean done = false; !done;) {
			try {
				System.out.print("Prezzo: ");
				prezzo = Prezzo.parse(menu.readline());
				done = true;
			} catch (IOException e) {
				System.out.println("Try again");
			} catch (IllegalArgumentException e) {
				System.out.println(e.getMessage());
			}
		}
		return prezzo;
	}
	private void aggiungiAutomobile() {
		System.out.println("Aggiunta automobile");
		System.out.println("--------------------");
		Targa targa = leggiTarga();
		CasaProduttrice casa = leggiCasaProduttrice();
		ModelloVeicolo nome = leggiModelloVeicolo();
		Prezzo prezzo = leggiPrezzoVeicolo();
		concessionaria.aggiungiAutomobile(targa, casa, nome, prezzo);
	}

	private void aggiungiMoto() {
		System.out.println("Aggiunta moto");
		System.out.println("--------------");
		Targa targa = leggiTarga();
		CasaProduttrice casa = leggiCasaProduttrice();
		ModelloVeicolo nome = leggiModelloVeicolo();
		Prezzo prezzo = leggiPrezzoVeicolo();
		concessionaria.aggiungiMoto(targa, casa, nome, prezzo);
	}

	private void stampaListaVeicoli(List<Veicolo> veicoli) {
		System.out.println(String.format("%-10s\t%-20s\t%-10s\t%-40s\t%-40s\t%-15s", "ID", "TIPO", "TARGA", "CASA PRODUTTRICE", "MODELLO", "PREZZO"));
		for(Veicolo v : veicoli) {
			System.out.println(v);
		}
	}

	private void stampaListaVeicoli() {
		System.out.println("Parco veicoli");
		System.out.println("--------------");
		stampaListaVeicoli(concessionaria.veicoli());
	}

	private void stampaValoreParcoVeicoli() {
		System.out.println("Valore parco veicoli");
		System.out.println("---------------------");
		System.out.println(concessionaria.calcolaValore());
	}
	
	// FIXME: if this service is exposed on the web, we should restrict to specific paths
	private void salva() {
		System.out.println("Salvataggio su file");
		System.out.println("--------------------");
		for(boolean done = false; !done;) {
			try {
				System.out.print("Filename: ");
				String line = menu.readline();
				concessionaria.salvaSuFile(line);
				done = true;
			} catch (IOException | IllegalStateException e) {
				System.out.println(e.getMessage());
			}
		}
		System.out.println("Salvataggio completato!");
	}
	
	// FIXME: if this service is exposed on the web, we should restrict to specific paths
	private void carica() {
		System.out.println("Caricamento da file");
		System.out.println("--------------------");
		for(boolean done = false; !done;) {
			try {
				System.out.print("Filename: ");
				String line = menu.readline();
				concessionaria.leggiDaFile(line);
				done = true;
			} catch (IOException | IllegalStateException e) {
				System.out.println(e.getMessage());
			}
		}
		System.out.println("Caricamento completato!");
	}
	
	private Veicolo selezionaVeicolo() {
		stampaListaVeicoli();
		Veicolo veicolo = null;
		for(boolean done = false; !done;) {
			try {
				System.out.print("Inserire ID (0 per annullare): ");
				String line = menu.readline();
				if(line.equals("0")) break;
				Id id = Id.parse(line);
				veicolo = concessionaria.selezionaVeicolo(id);
				done = true;
			} catch (IOException | IllegalArgumentException e) {
				System.out.println(e.getMessage());
			}
		}
		return veicolo;
	}
	
	private void modificaVeicolo() {
		Veicolo veicolo = selezionaVeicolo();
		if(veicolo == null) return;
		System.out.println("Valore corrente: " + veicolo.targa());
		veicolo.changeTarga(leggiTarga());
		System.out.println("Valore corrente:" + veicolo.casaProduttrice());
		veicolo.changeCasaProduttrice(leggiCasaProduttrice());
		System.out.println("Valore corrente: " + veicolo.modello());
		veicolo.changeModello(leggiModelloVeicolo());
		System.out.println("Valore corrente: " + veicolo.prezzo());
		veicolo.changePrezzo(leggiPrezzoVeicolo());
		System.out.println("Modifica completata!");
	}
	
	public void rimuoviVeicolo() {
		Veicolo veicolo = selezionaVeicolo();
		if(veicolo == null) return;
		concessionaria.rimuoviVeicolo(veicolo);
		System.out.println("Rimozione completata!");
	}
	
	public void stampaListaCaseProduttriciDiAuto() {
		System.out.println("Case produttrici di auto");
		System.out.println("-------------------------");
		Set<CasaProduttrice> res = new HashSet<CasaProduttrice>();
		for(Veicolo v : concessionaria.veicoli()) {
			if(v.getClass().equals(Automobile.class)) res.add(v.casaProduttrice());
		}
		for(CasaProduttrice c : res) {
			System.out.println(c);
		}
	}
	
	public void stampaListaCaseProduttriciDiMoto() {
		System.out.println("Case produttrici di moto");
		System.out.println("-------------------------");
		Set<CasaProduttrice> res = new HashSet<CasaProduttrice>();
		for(Veicolo v : concessionaria.veicoli()) {
			if(v.getClass().equals(Moto.class)) res.add(v.casaProduttrice());
		}
		for(CasaProduttrice c : res) {
			System.out.println(c);
		}
	}
	
	public void stampaListaAutoDiCasaProduttrice() {
		CasaProduttrice casa = leggiCasaProduttrice();
		List<Veicolo> veicoli = new ArrayList<Veicolo>();
		for(Veicolo v : concessionaria.veicoli()) {
			if(v.getClass().equals(Automobile.class) && v.casaProduttrice().equals(casa)) {
				veicoli.add(v);
			}
		}
		stampaListaVeicoli(veicoli);
	}
	
	public void stampaListaMotoDiCasaProduttrice() {
		CasaProduttrice casa = leggiCasaProduttrice();
		List<Veicolo> veicoli = new ArrayList<Veicolo>();
		for(Veicolo v : concessionaria.veicoli()) {
			if(v.getClass().equals(Moto.class) && v.casaProduttrice().equals(casa)) {
				veicoli.add(v);
			}
		}
		stampaListaVeicoli(veicoli);
	}
	
	public void run() {
		menu.run();
	}
	
	public static void main(String[] args) {
		try {
			AppConcessionaria mc = new AppConcessionaria();
			mc.run();
		} catch (Exception e) {
			System.out.println("Panic error!");
		}
	}
}
