package net.alviano.safe;

import static org.apache.commons.lang3.Validate.*;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class Id {
	private static final Pattern parseRegex = Pattern.compile("\\d{1,15}");
	
	public static Id parse(String value) {
		notBlank(value);
		Matcher matcher = parseRegex.matcher(value);
		if(!matcher.matches()) throw new IllegalArgumentException("Id: Errore di parsing");
		return new Id(Long.parseLong(value));
	}
	
	public static Id max(Id a, Id b) {
		return a.value > b.value ? a : b;
	}
	
	private final long value;
	
	public Id(long value) {
		inclusiveBetween(1, 1000000000000000L-1, value);
		this.value = value;
	}
	
	public long value() {
		return this.value;
	}
	
	public Id next() {
		return new Id(value + 1);
	}
	
	@Override
	public String toString() {
		return "" + value;
	}
	
	@Override
	public boolean equals(Object o) {
		if(o == null) return false;
		if(o.getClass() != Id.class) return false;
		Id i = (Id) o;
		return this.value == i.value;
	}
	
	@Override
	public int hashCode() {
		return Long.hashCode(this.value);
	}
}
