package net.alviano.safe;

import static org.apache.commons.lang3.Validate.*;

public abstract class Veicolo {
	private final Id id;
	private Targa targa;
	private CasaProduttrice casaProduttrice;
	private ModelloVeicolo modello;
	private Prezzo prezzo;
	
	public Veicolo(Id id, Targa targa, CasaProduttrice casaProduttrice, ModelloVeicolo modello, Prezzo prezzo) {
		this.id = notNull(id);
		this.targa = notNull(targa);
		this.casaProduttrice = notNull(casaProduttrice);
		this.modello = notNull(modello);
		this.prezzo = notNull(prezzo);
	}
	
	public String tipo() {
		return this.getClass().getSimpleName();
	}
	
	public String toString() {
		return String.format("%10s\t%-20s\t%-10s\t%-40s\t%-40s\t%15s", id, tipo(), targa, casaProduttrice, modello, calcolaPrezzoFinale());
	}
	
	public String toSaveFormat() {
		return String.format("%s;%s;%s;%s;%s;%s", id, tipo(), targa, casaProduttrice, modello, prezzo);
	}
	
	public Id id() {
		return id;
	}
	
	public Targa targa() {
		return targa;
	}
	
	public void changeTarga(Targa value) {
		targa = notNull(value);
	}
	
	public CasaProduttrice casaProduttrice() {
		return casaProduttrice;
	}
	
	public void changeCasaProduttrice(CasaProduttrice value) {
		casaProduttrice = notNull(value);
	}
	
	public ModelloVeicolo modello() {
		return modello;
	}
	
	public void changeModello(ModelloVeicolo value) {
		modello = notNull(value);
	}
	
	public Prezzo prezzo() {
		return prezzo;
	}
	
	public void changePrezzo(Prezzo value) {
		prezzo = notNull(value);
	}
	
	public Prezzo calcolaPrezzoFinale() {
		return prezzo;
	}
	
	// Improvement by Salvatore Laiso: Let's get a compilation error if we forget to define equals() in subclasses. Let's promote fail-fast approaches.
	@Override
//	public boolean equals(Object obj) {
//		throw new IllegalStateException("Subclasses of Veicolo must define equals()");
//	}
	public abstract boolean equals(Object obj);
	
	@Override
	public abstract int hashCode();
}
